/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998-2000  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    Schrack-Seconet/TU Wien:
    Functions for AES GCM-mode de/encryption. Used in auth.c and lfd_encrypt.c
 */

/*
 * schrack_aes-gcm_crypt; GG
 */ 
#ifndef _SCHRACK_AES_INC_
#define _SCHRACK_AES_INC_

#ifndef __APPLE_CC__
#include <polarssl/entropy.h>
#include <polarssl/ctr_drbg.h>
#include <polarssl/md5.h>
#include <polarssl/gcm.h>
#else
#include <sys/md5.h>
#include <crypto/gcm.h>
#endif  /* __APPLE_CC__ */

#include "SecoNet_AES_defs.h"

/* Generic helper functions ***************************************************/

int AES_GetError( void );

BOOL AES_SetKeyHex( const char* sKeyHex );
BOOL AES_SetKeyPassword( const char* sPassword );

/* Random number generation ***************************************************/

int RAND_bytes( uint8* bytes, int count );


/* AES en/decryption **********************************************************/

typedef aes_context TCtxAES;

typedef struct
{
   uint16 seqNum;
} TAddData;

int AES_InitEncrypt( void );
int AES_InitDecrypt( void );
void AES_Exit( void );
int AES_EncryptECB( char* aInput, char* aOutput );
int AES_DecryptECB( char* aInput, char* aOutput );


/* AES-GCM en/decryption ******************************************************/

typedef gcm_context TCtxGCM;

int AESGCM_Init( void );
void AESGCM_Exit( void );
int AESGCM_Encrypt( char* buffer, int length, char** pOutput );
int AESGCM_Decrypt( char* buffer, int length, char** pDecoded );
int AESGCM_DecryptAuth( char* buffer, int length, char** pDecoded );


#endif // _SCHRACK_AES_INC_
