/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998-2000  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    SecoNet:
    Definitions for AES mode and types.
 */

#ifndef _SECONET_AES_DEFS_
#define _SECONET_AES_DEFS_

typedef unsigned char uint8;
typedef unsigned short uint16;
typedef int BOOL;
#define TRUE 1
#define FALSE 0

/* Bit sizes of block and initialization vector *******************************/

#define AES_BLOCKSIZE_BITS 128
#define AES_BLOCK_LENGTH (AES_BLOCKSIZE_BITS>>3)

#define AES_KEYSIZE_BITS AES_BLOCKSIZE_BITS
#define AES_KEY_LENGTH (AES_KEYSIZE_BITS>>3)
   // Note: MD5 is used to hash the AES key from the password in HashKey().
   // If blocksize is to be changed, key generation must be adapted there.

#define AESGCM_BLOCKSIZE_BITS AES_BLOCKSIZE_BITS
#define AESGCM_BLOCK_LENGTH (AESGCM_BLOCKSIZE_BITS>>3)

#define AESGCM_KEYSIZE_BITS AESGCM_BLOCKSIZE_BITS
#define AESGCM_KEY_LENGTH (AESGCM_KEYSIZE_BITS>>3)

#define AESGCM_IV_BITSIZE 96
#define AESGCM_IV_LENGTH (AESGCM_IV_BITSIZE>>3)
   // IV of 12 bytes/96 bits, the remainder to 128 is used as counter

#define MINIMUM_AESKEY_LENGTH AES_KEY_LENGTH
	// Minimum length a user-provided key must have, the rest will be padded with
	// zeroes (to guarantee agreement on server and client side). For security
	// reasons, this should be set to the standard AES key length.

/* Sizes of fields in AES header **********************************************/

// Additional Authenticated Data fields:
#define AESGCM_AAD_SEQ_LENGTH 2 /* Sequence number for the packet */
#define AESGCM_AAD_LENGTH (AESGCM_AAD_SEQ_LENGTH) /* total AAD length */

#define AESGCM_TAG_LENGTH 8 /* use 8 bytes of the authentication tag */

#define AESGCM_BUFFER_TRAIL 8
	// When decoding into the same buffer, output must be offset by 8 bytes


// Header is composed of AAD, TAG, and IV.
#define AESGCM_HEADER_LENGTH (AESGCM_AAD_LENGTH + AESGCM_TAG_LENGTH + AESGCM_IV_LENGTH )

#define AESGCM_ENCRYPT GCM_ENCRYPT
#define AESGCM_DECRYPT GCM_DECRYPT


#endif // _SECONET_AES_DEFS_
